# frozen_string_literal: true

module Bundler
  module Security
    module Voting
      # Module responsible for building unsafe gem details
      module BuildUnsafeGem
        # Differ url
        DIFF_URL = 'https://diff.coditsu.io/gems'

        class << self
          # Builds details of an unsafe gem
          #
          # @param gem_policy [Voting::GemPolicy]
          #
          # @return [String]
          def call(gem_policy)
            [
              gem_policy.name,
              gem_policy.new_version? ? gem_policy.new_version : gem_policy.current_version,
              '-',
              [
                approved(gem_policy.remote_policy, gem_policy),
                rejected(gem_policy.remote_policy, gem_policy)
              ].compact.join(', ')
            ].join(' ')
          end

          # Builds safe details
          #
          # @param remote_policy [Voting::RemotePolicy]
          # @param gem_policy [Voting::GemPolicy]
          #
          # @return [String]
          def approved(remote_policy, gem_policy)
            return if gem_policy.approved?

            array = []
            array << "approved (#{gem_policy.approved} expected #{remote_policy.approved})"
            array << differ_url(gem_policy)
            array.join(' ')
          end

          # Builds malicious details
          #
          # @param remote_policy [Voting::RemotePolicy]
          # @param gem_policy [Voting::GemPolicy]
          #
          # @return [String]
          def rejected(remote_policy, gem_policy)
            return if gem_policy.rejected?

            array = []
            array << "rejected (#{gem_policy.rejected} expected #{remote_policy.rejected})"
            array << differ_url(gem_policy)
            array.join(' ')
          end

          # Builds differ url for gem with version details
          #
          # @param gem_policy [Voting::GemPolicy]
          #
          # @return [String]
          def differ_url(gem_policy)
            array = [DIFF_URL, gem_policy.name, gem_policy.current_version]
            array << gem_policy.new_version if gem_policy.new_version?
            array.join('/')
          end
        end
      end
    end
  end
end
