# frozen_string_literal: true

module Bundler
  module Security
    module Voting
      # Gem policy with statistics from Coditsu differ
      class GemPolicy
        attr_reader :name, :current_version, :new_version, :remote_policy

        # Build gem policy
        #
        # @param name [String] gem name
        # @param gem_data [Array] gem version and statistics from Coditsu
        # @param remote_policy [Voting::RemotePolicy]
        def initialize(name, gem_data, remote_policy)
          @name = name
          @new_version = nil

          versions = gem_data.first

          raise Errors::InvalidRemoteVersionsType, versions.class unless versions.is_a?(Array)

          @current_version = versions.first.empty? ? versions.last : versions.first
          @new_version = versions.last if @current_version != versions.last

          @remote_policy = remote_policy
          @threshold = gem_data.last[remote_policy.type]
        end

        # How many time gem was marked as safe
        #
        # @return [Integer]
        def approved
          @threshold['up'].to_i
        end

        # How many time gem was marked as malicious
        #
        # @return [Integer]
        def rejected
          @threshold['down'].to_i
        end

        # Checks if a gem is safe based on a remote policy
        #
        # @return [Boolean] true if it's safe, false otherwise
        def approved?
          approved >= @remote_policy.approved
        end

        # Checks if a gem is malicious based on a remote policy
        #
        # @return [Boolean] true if it's malicious, false otherwise
        def rejected?
          @remote_policy.rejected >= rejected
        end

        # Check if a new version was requested
        #
        # @return [Boolean] true if new version was requested, false otherwise
        def new_version?
          !@new_version.nil?
        end
      end
    end
  end
end
