# frozen_string_literal: true

require 'net/http'
require 'openssl'
require 'json'

module Bundler
  module Security
    module Voting
      # Module responsible for doing request to Coditsu differ
      module Request
        # Differ endpoint url
        ENDPOINT_URL = 'https://diff.coditsu.io/api/bundler.json'
        # Request headers
        HEADERS = { 'Content-Type': 'application/json' }.freeze

        private_constant :ENDPOINT_URL, :HEADERS

        class << self
          # Execute request to the differ
          #
          # @param config [OpenStruct] Coditsu config
          # @param payload [Hash] with versions to check
          #
          # @return [Net::HTTPResponse] response from Coditsu differ
          def call(config, payload)
            build_http do |http, uri|
              http.request(build_request(uri, config, payload))
            end
          end

          # Builds http connection object
          def build_http
            uri = URI(differ_url)

            Net::HTTP.start(
              uri.host,
              uri.port,
              use_ssl: uri.scheme == 'https',
              verify_mode: OpenSSL::SSL::VERIFY_NONE
            ) { |http| yield(http, uri) }
          end

          # Build http post request and assigns headers and payload
          #
          # @param uri [URI::HTTPS]
          # @param config [OpenStruct] Coditsu config
          # @param payload [Hash] with versions to check
          #
          # @return [Net::HTTP::Post]
          def build_request(uri, config, payload)
            Net::HTTP::Post
              .new(uri.request_uri, HEADERS)
              .tap { |request| assign_auth(request, config) }
              .tap { |request| assign_payload(request, payload) }
          end

          # Assigns basic authorization if provided in the config
          #
          # @param request [Net::HTTP::Post] prepared http post
          # @param config [OpenStruct] Coditsu config
          def assign_auth(request, config)
            return unless config
            return unless config.api_key
            return unless config.api_secret

            request.basic_auth(config.api_key, config.api_secret)
          end

          # Assigns payload as json
          #
          # @param request [Net::HTTP::Post] prepared http post
          # @param payload [Hash] with versions to check
          def assign_payload(request, payload)
            request.body = JSON.dump(payload)
          end

          # Provides differ endpoint url
          #
          # @return [String]
          def differ_url
            ENV['CODITSU_DIFFER_URL'] || ENDPOINT_URL
          end
        end
      end
    end
  end
end
