# frozen_string_literal: true

require 'json'

module Bundler
  module Security
    module Voting
      # Module responsible for handling both local and remote gem versions
      module Versions
        # Module responsible for fetching safe/malicious votes
        # for current or current/new versions of gems
        module Remote
          # Differ bundler url
          ENDPOINT_URL = 'https://diff.coditsu.io/api/bundler.json'

          private_constant :ENDPOINT_URL

          class << self
            # @param command [String] either install or update
            # @param definition [Bundler::Definition] definition for your source
            def call(command, definition)
              config = fetch_config

              Request
                .call(config, payload(command, config&.repository_id, definition))
                .then { |response| JSON.parse(response.body) }
            end

            # @param command [String] either install or update
            # @param repository_id [String] coditsu repository_id
            # @param definition [Bundler::Definition] definition for your source
            #
            # @return [Hash] payload for differ bundler endpoint
            def payload(command, repository_id, definition)
              Local.call(command, definition).each_with_object({}) do |(name, versions), hash|
                hash[:data] ||= {}
                hash[:data][:repository_id] = repository_id if repository_id
                hash[:data][:gems] ||= {}
                hash[:data][:gems][name] = versions
              end
            end

            # Fetch coditsu config file
            #
            # @return [OpenStruct, nil] configuration object
            #
            # @raise [Errors::MissingConfigurationFile] when no config file
            def fetch_config
              Config::Fetcher.call(
                File.expand_path('..', Bundler.bin_path)
              )
            rescue Errors::MissingConfigurationFile
              nil
            end
          end
        end
      end
    end
  end
end
